@@ -60,6 +60,12 @@ typedef enum
  * @GST_MIXER_MESSAGE_RECORD_TOGGLED: A record-toggled GstMixer message
  * @GST_MIXER_MESSAGE_VOLUME_CHANGED: A volume-changed GstMixer message
  * @GST_MIXER_MESSAGE_OPTION_CHANGED: An option-changed GstMixer message
+ * @GST_MIXER_MESSAGE_OPTIONS_LIST_CHANGED: An options-list-changed
+ *     GstMixer message, posted when the list of available options for a
+ *     GstMixerOptions object has changed (Since: 0.10.18)
+ * @GST_MIXER_MESSAGE_MIXER_CHANGED: A mixer-changed GstMixer message, posted
+ *     when the list of available mixer tracks has changed. The application
+ *     should re-build its interface in this case (Since: 0.10.18)
  * 
  * An enumeration for the type of a GstMixer message received on the bus
  *
@@ -71,7 +77,9 @@ typedef enum
   GST_MIXER_MESSAGE_MUTE_TOGGLED,
   GST_MIXER_MESSAGE_RECORD_TOGGLED,
   GST_MIXER_MESSAGE_VOLUME_CHANGED,
-  GST_MIXER_MESSAGE_OPTION_CHANGED
+  GST_MIXER_MESSAGE_OPTION_CHANGED,
+  GST_MIXER_MESSAGE_OPTIONS_LIST_CHANGED,
+  GST_MIXER_MESSAGE_MIXER_CHANGED
 } GstMixerMessageType;
 
 /**
@@ -79,16 +87,24 @@ typedef enum
  * @GST_MIXER_FLAG_NONE: No flags
  * @GST_MIXER_FLAG_AUTO_NOTIFICATIONS: The mixer implementation automatically sends
  *    notification messages.
+ * @GST_MIXER_FLAG_HAS_WHITELIST: The mixer implementation flags tracks that should
+ *    be displayed by default (whitelisted). Since: 0.10.23
+ *
+ * @GST_MIXER_FLAG_GROUPING: The mixer implementation will leave some controls
+ *    marked without either input or output.  Controls marked as input or
+ *    output should be grouped with input & output sliders, even if they
+ *    are options or bare switches. Since: 0.10.23
  * 
- * Flags for supported features. Whether the element automatically sends 
- * notifications on the bus is the only one for now. 
+ * Flags for supported features.
  *
  * Since: 0.10.14
  */
 typedef enum
 {
   GST_MIXER_FLAG_NONE                = 0,
-  GST_MIXER_FLAG_AUTO_NOTIFICATIONS  = (1<<0)
+  GST_MIXER_FLAG_AUTO_NOTIFICATIONS  = (1<<0),
+  GST_MIXER_FLAG_HAS_WHITELIST       = (1<<1),
+  GST_MIXER_FLAG_GROUPING            = (1<<2),
 } GstMixerFlags;
 
 struct _GstMixerClass {
@@ -183,6 +199,12 @@ void            gst_mixer_option_changed (GstMixer      *mixer,
                                           GstMixerOptions *opts,
                                           gchar         *value);
 
+void            gst_mixer_mixer_changed   (GstMixer        *mixer);
+
+void            gst_mixer_options_list_changed (GstMixer        *mixer,
+                                                GstMixerOptions *opts);
+
+
 GstMixerFlags   gst_mixer_get_mixer_flags (GstMixer *mixer);
 
 /* Functions for recognising and parsing GstMixerMessages on the bus */
@@ -200,6 +222,8 @@ void            gst_mixer_message_parse_volume_changed (GstMessage *message,
 void            gst_mixer_message_parse_option_changed (GstMessage *message,
                                                         GstMixerOptions **options,
                                                         const gchar **value);
+void            gst_mixer_message_parse_options_list_changed (GstMessage *message,
+                                                              GstMixerOptions **options);
 
 G_END_DECLS
 
