@@ -1,8 +1,7 @@
 /* GStreamer Navigation
  * Copyright (C) 2003 Ronald Bultje <rbultje@ronald.bitfreak.net>
- * Copyright (C) 2007-2009 Jan Schmidt <thaytan@noraisin.net>
  *
- * navigation.c: navigation event virtual class function wrappers
+ * navigation.c: navigation design virtual class function wrappers
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -20,45 +19,6 @@
  * Boston, MA 02111-1307, USA.
  */
 
-/**
- * SECTION:gstnavigation
- * @short_description: Interface for creating, sending and parsing navigation
- * events.
- *
- * The Navigation interface is used for creating and injecting navigation related
- * events such as mouse button presses, cursor motion and key presses. The associated
- * library also provides methods for parsing received events, and for sending and
- * receiving navigation related bus events. One main usecase is DVD menu navigation.
- *
- * The main parts of the API are:
- * <itemizedlist>
- * <listitem>
- * <para>
- * The GstNavigation interface, implemented by elements which provide an application
- * with the ability to create and inject navigation events into the pipeline.
- * </para>
- * </listitem>
- * <listitem>
- * <para>
- * GstNavigation event handling API. GstNavigation events are created in response to
- * calls on a GstNavigation interface implementation, and sent in the pipeline. Upstream
- * elements can use the navigation event API functions to parse the contents of received
- * messages.
- * </para>
- * </listitem>
- * <listitem>
- * <para>
- * GstNavigation message handling API. GstNavigation messages may be sent on the message
- * bus to inform applications of navigation related changes in the pipeline, such as the
- * mouse moving over a clickable region, or the set of available angles changing.
- * </para><para>
- * The GstNavigation message functions provide functions for creating and parsing
- * custom bus messages for signaling GstNavigation changes.
- * </para>
- * </listitem>
- * </itemizedlist>
- */
-
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
@@ -72,8 +32,6 @@ static void gst_navigation_class_init (GstNavigationInterface * iface);
 #define GST_NAVIGATION_QUERY_NAME "GstNavigationQuery"
 #define GST_NAVIGATION_EVENT_NAME "application/x-gst-navigation"
 
-#define WARN_IF_FAIL(exp,msg) if(G_UNLIKELY(!(exp))){g_warning("%s",(msg));}
-
 GType
 gst_navigation_get_type (void)
 {
@@ -119,7 +77,7 @@ gst_navigation_send_event (GstNavigation * navigation, GstStructure * structure)
 }
 
 /**
- * gst_navigation_send_key_event:
+ * gst_navigation_send_mouse_event:
  * @navigation: The navigation interface instance
  * @event: The type of the key event. Recognised values are "key-press" and
  * "key-release"
@@ -457,21 +415,20 @@ gst_navigation_query_parse_angles (GstQuery * query, guint * cur_angle,
     guint * n_angles)
 {
   GstStructure *structure;
-  gboolean ret = TRUE;
 
   g_return_val_if_fail (GST_NAVIGATION_QUERY_HAS_TYPE (query, ANGLES), FALSE);
 
   structure = gst_query_get_structure (query);
 
   if (cur_angle)
-    ret &= gst_structure_get_uint (structure, "angle", cur_angle);
+    g_return_val_if_fail (gst_structure_get_uint (structure,
+            "angle", cur_angle), FALSE);
 
   if (n_angles)
-    ret &= gst_structure_get_uint (structure, "angles", n_angles);
+    g_return_val_if_fail (gst_structure_get_uint (structure,
+            "angles", n_angles), FALSE);
 
-  WARN_IF_FAIL (ret, "Couldn't extract details from angles query");
-
-  return ret;
+  return TRUE;
 }
 
 /* Navigation Messages */
@@ -486,7 +443,7 @@ gst_navigation_query_parse_angles (GstQuery * query, guint * cur_angle,
  * Check a bus message to see if it is a #GstNavigation event, and return
  * the #GstNavigationMessageType identifying the type of the message if so.
  *
- * Returns: The type of the #GstMessage, or
+ * Returns: The type of the #GstNavigationMessage, or
  * #GST_NAVIGATION_MESSAGE_INVALID if the message is not a #GstNavigation
  * notification.
  *
@@ -647,21 +604,20 @@ gst_navigation_message_parse_angles_changed (GstMessage * message,
     guint * cur_angle, guint * n_angles)
 {
   const GstStructure *s;
-  gboolean ret = TRUE;
 
   g_return_val_if_fail (GST_NAVIGATION_MESSAGE_HAS_TYPE (message,
           ANGLES_CHANGED), FALSE);
 
   s = gst_message_get_structure (message);
   if (cur_angle)
-    ret &= gst_structure_get_uint (s, "angle", cur_angle);
+    g_return_val_if_fail (gst_structure_get_uint (s, "angle", cur_angle),
+        FALSE);
 
   if (n_angles)
-    ret &= gst_structure_get_uint (s, "angles", n_angles);
+    g_return_val_if_fail (gst_structure_get_uint (s, "angles", n_angles),
+        FALSE);
 
-  WARN_IF_FAIL (ret, "Couldn't extract details from angles-changed event");
-
-  return ret;
+  return TRUE;
 }
 
 #define GST_NAVIGATION_EVENT_HAS_TYPE(event,event_type) \
@@ -752,9 +708,6 @@ gst_navigation_event_parse_key_event (GstEvent * event, const gchar ** key)
  * a mouse button release event. Determine which type the event is using
  * gst_navigation_event_get_type() to retrieve the #GstNavigationEventType.
  *
- * Returns: TRUE if the button number and both coordinates could be extracted,
- *     otherwise FALSE.
- *
  * Since: 0.10.23
  */
 gboolean
@@ -763,7 +716,6 @@ gst_navigation_event_parse_mouse_button_event (GstEvent * event, gint * button,
 {
   GstNavigationEventType e_type;
   const GstStructure *s;
-  gboolean ret = TRUE;
 
   e_type = gst_navigation_event_get_type (event);
   g_return_val_if_fail (e_type == GST_NAVIGATION_EVENT_MOUSE_BUTTON_PRESS ||
@@ -771,15 +723,13 @@ gst_navigation_event_parse_mouse_button_event (GstEvent * event, gint * button,
 
   s = gst_event_get_structure (event);
   if (x)
-    ret &= gst_structure_get_double (s, "pointer_x", x);
+    g_return_val_if_fail (gst_structure_get_double (s, "pointer_x", x), FALSE);
   if (y)
-    ret &= gst_structure_get_double (s, "pointer_y", y);
+    g_return_val_if_fail (gst_structure_get_double (s, "pointer_y", y), FALSE);
   if (button)
-    ret &= gst_structure_get_int (s, "button", button);
+    g_return_val_if_fail (gst_structure_get_int (s, "button", button), FALSE);
 
-  WARN_IF_FAIL (ret, "Couldn't extract details from mouse button event");
-
-  return ret;
+  return TRUE;
 }
 
 /**
@@ -791,8 +741,6 @@ gst_navigation_event_parse_mouse_button_event (GstEvent * event, gint * button,
  * Inspect a #GstNavigation mouse movement event and extract the coordinates
  * of the event.
  *
- * Returns: TRUE if both coordinates could be extracted, otherwise FALSE.
- *
  * Since: 0.10.23
  */
 gboolean
@@ -800,20 +748,17 @@ gst_navigation_event_parse_mouse_move_event (GstEvent * event, gdouble * x,
     gdouble * y)
 {
   const GstStructure *s;
-  gboolean ret = TRUE;
 
   g_return_val_if_fail (GST_NAVIGATION_EVENT_HAS_TYPE (event, MOUSE_MOVE),
       FALSE);
 
   s = gst_event_get_structure (event);
   if (x)
-    ret &= gst_structure_get_double (s, "pointer_x", x);
+    g_return_val_if_fail (gst_structure_get_double (s, "pointer_x", x), FALSE);
   if (y)
-    ret &= gst_structure_get_double (s, "pointer_y", y);
+    g_return_val_if_fail (gst_structure_get_double (s, "pointer_y", y), FALSE);
 
-  WARN_IF_FAIL (ret, "Couldn't extract positions from mouse move event");
-
-  return ret;
+  return TRUE;
 }
 
 /**
@@ -825,8 +770,6 @@ gst_navigation_event_parse_mouse_move_event (GstEvent * event, gdouble * x,
  * Inspect a #GstNavigation command event and retrieve the enum value of the
  * associated command.
  *
- * Returns: TRUE if the navigation command could be extracted, otherwise FALSE.
- *
  * Since: 0.10.23
  */
 gboolean
@@ -834,15 +777,13 @@ gst_navigation_event_parse_command (GstEvent * event,
     GstNavigationCommand * command)
 {
   const GstStructure *s;
-  gboolean ret = TRUE;
-
   g_return_val_if_fail (GST_NAVIGATION_EVENT_HAS_TYPE (event, COMMAND), FALSE);
 
   if (command) {
     s = gst_event_get_structure (event);
-    ret = gst_structure_get_uint (s, "command-code", (guint *) command);
-    WARN_IF_FAIL (ret, "Couldn't extract command code from command event");
+    g_return_val_if_fail (gst_structure_get_uint (s, "command-code", command),
+        FALSE);
   }
 
-  return ret;
+  return TRUE;
 }
