@@ -88,40 +88,6 @@ gst_netbuffer_finalize (GstNetBuffer * nbuf)
   GST_MINI_OBJECT_CLASS (parent_class)->finalize (GST_MINI_OBJECT (nbuf));
 }
 
-/* this is copy'n'pasted from _gst_buffer_copy() in lack of a better solution;
- * keep in sync with _gst_buffer_copy() */
-static void
-notgst_buffer_copy_fields_in_place (GstBuffer * copy, GstBuffer * buffer)
-{
-  guint mask;
-
-  GST_LOG ("copying %p to %p", buffer, copy);
-
-  /* copy relevant flags */
-  mask = GST_BUFFER_FLAG_PREROLL | GST_BUFFER_FLAG_IN_CAPS |
-      GST_BUFFER_FLAG_DELTA_UNIT | GST_BUFFER_FLAG_DISCONT |
-      GST_BUFFER_FLAG_GAP;
-  GST_MINI_OBJECT_FLAGS (copy) |= GST_MINI_OBJECT_FLAGS (buffer) & mask;
-
-  /* we simply copy everything from our parent */
-  copy->data = g_memdup (buffer->data, buffer->size);
-  /* make sure it gets freed (even if the parent is subclassed, we return a
-     normal buffer) */
-  copy->malloc_data = copy->data;
-
-  copy->size = buffer->size;
-
-  GST_BUFFER_TIMESTAMP (copy) = GST_BUFFER_TIMESTAMP (buffer);
-  GST_BUFFER_DURATION (copy) = GST_BUFFER_DURATION (buffer);
-  GST_BUFFER_OFFSET (copy) = GST_BUFFER_OFFSET (buffer);
-  GST_BUFFER_OFFSET_END (copy) = GST_BUFFER_OFFSET_END (buffer);
-
-  if (GST_BUFFER_CAPS (buffer))
-    GST_BUFFER_CAPS (copy) = gst_caps_ref (GST_BUFFER_CAPS (buffer));
-  else
-    GST_BUFFER_CAPS (copy) = NULL;
-}
-
 static GstNetBuffer *
 gst_netbuffer_copy (GstNetBuffer * nbuf)
 {
@@ -129,15 +95,21 @@ gst_netbuffer_copy (GstNetBuffer * nbuf)
 
   copy = gst_netbuffer_new ();
 
-  /* can't just chain up to parent_class::copy() because that will allocate
-   * a normal GstBuffer for us, so we'd lose the to/from fields and we don't
-   * want that */
-  notgst_buffer_copy_fields_in_place (GST_BUFFER_CAST (copy),
-      GST_BUFFER_CAST (nbuf));
+  /* we simply copy everything from our parent */
+  GST_BUFFER_DATA (copy) =
+      g_memdup (GST_BUFFER_DATA (nbuf), GST_BUFFER_SIZE (nbuf));
+  /* make sure it gets freed (even if the parent is subclassed, we return a
+     normal buffer) */
+  GST_BUFFER_MALLOCDATA (copy) = GST_BUFFER_DATA (copy);
+  GST_BUFFER_SIZE (copy) = GST_BUFFER_SIZE (nbuf);
 
   memcpy (&copy->to, &nbuf->to, sizeof (nbuf->to));
   memcpy (&copy->from, &nbuf->from, sizeof (nbuf->from));
 
+  /* copy metadata */
+  gst_buffer_copy_metadata (GST_BUFFER_CAST (copy),
+      GST_BUFFER_CAST (nbuf), GST_BUFFER_COPY_ALL);
+
   return copy;
 }
 
@@ -218,7 +190,8 @@ gst_netaddress_get_net_type (GstNetAddress * naddr)
  * @address: a location to store the address.
  * @port: a location to store the port.
  *
- * Get the IPv4 address stored in @naddr into @address.
+ * Get the IPv4 address stored in @naddr into @address. This function requires
+ * that the address type of @naddr is of type #GST_NET_TYPE_IP4.
  *
  * Returns: TRUE if the address could be retrieved.
  */
@@ -228,7 +201,7 @@ gst_netaddress_get_ip4_address (GstNetAddress * naddr, guint32 * address,
 {
   g_return_val_if_fail (naddr != NULL, FALSE);
 
-  if (naddr->type == GST_NET_TYPE_UNKNOWN)
+  if (naddr->type == GST_NET_TYPE_UNKNOWN || naddr->type == GST_NET_TYPE_IP6)
     return FALSE;
 
   if (address)
@@ -247,21 +220,149 @@ gst_netaddress_get_ip4_address (GstNetAddress * naddr, guint32 * address,
  *
  * Get the IPv6 address stored in @naddr into @address.
  *
+ * If @naddr is of type GST_NET_TYPE_IP4, the transitional IP6 address is
+ * returned.
+ *
  * Returns: TRUE if the address could be retrieved.
  */
 gboolean
 gst_netaddress_get_ip6_address (GstNetAddress * naddr, guint8 address[16],
     guint16 * port)
 {
+  static guint8 ip4_transition[16] =
+      { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0xFF, 0xFF };
   g_return_val_if_fail (naddr != NULL, FALSE);
 
   if (naddr->type == GST_NET_TYPE_UNKNOWN)
     return FALSE;
 
-  if (address)
-    memcpy (address, naddr->address.ip6, 16);
+  if (address) {
+    if (naddr->type == GST_NET_TYPE_IP6) {
+      memcpy (address, naddr->address.ip6, 16);
+    } else {                    /* naddr->type == GST_NET_TYPE_IP4 */
+      memcpy (address, ip4_transition, 12);
+      memcpy (address + 12, (guint8 *) & (naddr->address.ip4), 4);
+    }
+  }
   if (port)
     *port = naddr->port;
 
   return TRUE;
 }
+
+/**
+ * gst_netaddress_get_address_bytes:
+ * @naddr: a network address
+ * @address: a location to store the result.
+ * @port: a location to store the port.
+ *
+ * Get just the address bytes stored in @naddr into @address.
+ *
+ * Returns: number of bytes actually copied
+ *
+ * Since: 0.10.22
+ */
+gint
+gst_netaddress_get_address_bytes (GstNetAddress * naddr, guint8 address[16],
+    guint16 * port)
+{
+  gint ret = 0;
+
+  g_return_val_if_fail (naddr != NULL, FALSE);
+
+  if (naddr->type == GST_NET_TYPE_UNKNOWN)
+    return 0;
+
+  if (address) {
+    if (naddr->type == GST_NET_TYPE_IP6) {
+      memcpy (address, naddr->address.ip6, 16);
+      ret = 16;
+    } else {                    /* naddr->type == GST_NET_TYPE_IP4 */
+      memcpy (address, (guint8 *) & (naddr->address.ip4), 4);
+      ret = 4;
+    }
+  }
+  if (port)
+    *port = naddr->port;
+
+  return ret;
+}
+
+/**
+ * gst_netaddress_set_address_bytes:
+ * @naddr: a network address
+ * @address: a location to store the result.
+ * @port: a location to store the port.
+ *
+ * Set just the address bytes stored in @naddr into @address.
+ *
+ * Returns: number of bytes actually copied
+ *
+ * Since: 0.10.22
+ */
+gint
+gst_netaddress_set_address_bytes (GstNetAddress * naddr, GstNetType type,
+    guint8 address[16], guint16 port)
+{
+  gint len = 0;
+
+  g_return_val_if_fail (naddr != NULL, 0);
+
+  naddr->type = type;
+  switch (naddr->type) {
+    case GST_NET_TYPE_UNKNOWN:
+    case GST_NET_TYPE_IP6:
+      len = 16;
+      memcpy (naddr->address.ip6, address, 16);
+      break;
+    case GST_NET_TYPE_IP4:
+      len = 4;
+      memcpy ((guint8 *) & (naddr->address.ip4), address, 4);
+      break;
+  }
+
+  if (port)
+    naddr->port = port;
+
+  return len;
+}
+
+/**
+ * gst_netaddress_equal:
+ * @naddr1: The first #GstNetAddress
+ * @naddr2: The second #GstNetAddress
+ *
+ * Compare two #GstNetAddress structures
+ *
+ * Returns: TRUE if they are identical, FALSE otherwise
+ *
+ * Since: 0.10.18
+ */
+gboolean
+gst_netaddress_equal (const GstNetAddress * naddr1,
+    const GstNetAddress * naddr2)
+{
+  g_return_val_if_fail (naddr1 != NULL, FALSE);
+  g_return_val_if_fail (naddr2 != NULL, FALSE);
+
+  if (naddr1->type != naddr2->type)
+    return FALSE;
+
+  if (naddr1->port != naddr2->port)
+    return FALSE;
+
+  switch (naddr1->type) {
+    case GST_NET_TYPE_IP4:
+      if (naddr1->address.ip4 != naddr2->address.ip4)
+        return FALSE;
+      break;
+    case GST_NET_TYPE_IP6:
+      if (memcmp (naddr1->address.ip6, naddr2->address.ip6,
+              sizeof (naddr1->address.ip6)))
+        return FALSE;
+      break;
+    default:
+      break;
+  }
+  return TRUE;
+}
