@@ -58,12 +58,32 @@
 
 #include "gstrtspurl.h"
 
+static void
+register_rtsp_url_type (GType * id)
+{
+  *id = g_boxed_type_register_static ("GstRTSPUrl",
+      (GBoxedCopyFunc) gst_rtsp_url_copy, (GBoxedFreeFunc) gst_rtsp_url_free);
+}
+
+GType
+gst_rtsp_url_get_type (void)
+{
+  static GType id;
+  static GOnce once = G_ONCE_INIT;
+
+  g_once (&once, (GThreadFunc) register_rtsp_url_type, &id);
+  return id;
+}
+
+
 #define RTSP_PROTO      "rtsp://"
 #define RTSP_PROTO_LEN  7
 #define RTSPU_PROTO     "rtspu://"
 #define RTSPU_PROTO_LEN 8
 #define RTSPT_PROTO     "rtspt://"
 #define RTSPT_PROTO_LEN 8
+#define RTSPH_PROTO     "rtsph://"
+#define RTSPH_PROTO_LEN 8
 
 /* format is rtsp[u]://[user:passwd@]host[:port]/abspath[?query] */
 
@@ -100,6 +120,9 @@ gst_rtsp_url_parse (const gchar * urlstr, GstRTSPUrl ** url)
   } else if (g_str_has_prefix (p, RTSPT_PROTO)) {
     res->transports = GST_RTSP_LOWER_TRANS_TCP;
     p += RTSPT_PROTO_LEN;
+  } else if (g_str_has_prefix (p, RTSPH_PROTO)) {
+    res->transports = GST_RTSP_LOWER_TRANS_HTTP | GST_RTSP_LOWER_TRANS_TCP;
+    p += RTSPH_PROTO_LEN;
   } else
     goto invalid;
 
@@ -176,6 +199,37 @@ invalid:
   }
 }
 
+/**
+ * gst_rtsp_url_copy:
+ * @url: a #GstRTSPUrl
+ *
+ * Make a copy of @url.
+ *
+ * Returns: a copy of @url. Free with gst_rtsp_url_free () after usage.
+ *
+ * Since: 0.10.22
+ */
+GstRTSPUrl *
+gst_rtsp_url_copy (const GstRTSPUrl * url)
+{
+  GstRTSPUrl *res;
+
+  g_return_val_if_fail (url != NULL, NULL);
+
+  res = g_new0 (GstRTSPUrl, 1);
+
+  res->transports = url->transports;
+  res->family = url->family;
+  res->user = g_strdup (url->user);
+  res->passwd = g_strdup (url->passwd);
+  res->host = g_strdup (url->host);
+  res->port = url->port;
+  res->abspath = g_strdup (url->abspath);
+  res->query = g_strdup (url->query);
+
+  return res;
+}
+
 /**
  * gst_rtsp_url_free:
  * @url: a #GstRTSPUrl
@@ -225,7 +279,7 @@ gst_rtsp_url_set_port (GstRTSPUrl * url, guint16 port)
  * Returns: #GST_RTSP_OK.
  */
 GstRTSPResult
-gst_rtsp_url_get_port (GstRTSPUrl * url, guint16 * port)
+gst_rtsp_url_get_port (const GstRTSPUrl * url, guint16 * port)
 {
   g_return_val_if_fail (url != NULL, GST_RTSP_EINVAL);
   g_return_val_if_fail (port != NULL, GST_RTSP_EINVAL);
@@ -248,7 +302,7 @@ gst_rtsp_url_get_port (GstRTSPUrl * url, guint16 * port)
  * Returns: a string with the request URI. g_free() after usage.
  */
 gchar *
-gst_rtsp_url_get_request_uri (GstRTSPUrl * url)
+gst_rtsp_url_get_request_uri (const GstRTSPUrl * url)
 {
   gchar *uri;
 
